#!/usr/bin/env python
# coding: utf-8

# In[1]:


import numpy as np
import matplotlib.pyplot as plt


# In[2]:


#Das Euler-Polygonzugverfahren im 1-Dimensionalen:
#Startwerte x0,y0, Reichweite b, Anzahl Schritte n
def polygonzug(x0,y0,b,n,F):
    h=(b-x0)/n # setzte Schrittweite fest
    x=[x0]     # speichere Stützstellen und starte mit Startwerten
    y=[y0]
    for i in range(n):
        # füge in jedem Schritt die neuen Stützstellen hinzu
        y.append(y[-1]+h*F(x[-1],y[-1]))
        x.append(x[-1]+h)
    return x,y


# In[3]:


#unsere DGL
def F(x,y):
    return x+y;


# In[4]:


#die echte Lösung des AWP für Startwerte x0=0, y0=1:
def u(x):
    return 2*np.exp(x)-x-1;


# In[5]:


#Schrittweite und Anzahl festlegen:
n=10
b=2

# plotte den Polygonzug
x,y = polygonzug(0,1,b,n,F)
plt.plot(x,y,color='blue')

# im Vergleich unsere echte Lösung
t=np.linspace(0,b,100)
plt.plot(t,u(t),color='red')

# das drum rum um den Plot
plt.title('Polygonzugverfahren für das AWP y\'=x+y mit y(0)=1')
plt.xlabel('X-Achse')
plt.ylabel('Y-Achse')
plt.grid(True)

plt.show()


# In[6]:


print('Unsere Stützstellen der Euler-Interpolation:')
for i in range(len(x)):
    print('x= ',x[i], '    y=', y[i])


# In[7]:


#explizite Fehlerabschätzung an den Stützstellen
delta=[]
for i in range(len(x)):
    delta.append(u(x[i])-y[i])
for i in range(len(delta)):
    print(delta[i])


# In[8]:


# ein weiteres Beispiel
def F2(x,y):
    return -y

#mit exakter Lösung für x0=0,y0=1:
def v(t):
    return np.exp(-t)


# In[9]:


#Schrittweite und Anzahl festlegen:
#je nach Schrittweite h<2 bzw. h>2 divergiert oder konvergiert der Polygonzug
n=10
b=20

#plotte den Polygonzug
x,y = polygonzug(0,1,b,n,F2)
plt.plot(x,y,color='blue')


# im Vergleich unsere echte Lösung
t=np.linspace(0,b,100)
plt.plot(t,v(t),color='red')

plt.title('Polygonzugverfahren für das AWP y\'=x+y mit y(0)=1')
plt.xlabel('X-Achse')
plt.ylabel('Y-Achse')
plt.grid(True)

plt.show()

